//
// OpenIrekia v2.0 Cliente Android
//
// Copyright 2009-2011 eFaber, S.L.
// Copyright 2009-2011 Ejie, S.A.
// Copyrigth 2009-2011 Dirección de Gobierno Abierto y Comunicación en Internet; 
//    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
//    Gobierno Vasco – Eusko Jaurlaritza 
// Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
// por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
// Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
// copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
// Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
// el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
// SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
// Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
// que establece la Licencia
//
//  http://open.irekia.net, openirekia@efaber.net

package net.efaber.irekia;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextWatcher;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;
import net.efaber.irekia.activities.About_activity;
import net.efaber.irekia.activities.Restarter_activity;
import net.efaber.irekia.activities.Settings_activity;
import net.efaber.irekia.childcontroller.Child_list_activity;
import net.efaber.irekia.login.Login_web_activity;
import net.efaber.irekia.models.Content_item;
import net.efaber.irekia.models.Section_state;
import net.efaber.irekia.models.paginatedresult.PaginatedResult;
import net.efaber.irekia.social.Social_Activity;
import net.efaber.irekia.social.answer.AnswerTask;
import net.efaber.irekia.social.argument.ArgumentTask;
import net.efaber.irekia.social.comment.CommentTask;
import net.efaber.irekia.social.create.StepController;
import net.efaber.irekia.social.vote.VoteTask;
import net.efaber.irekia.zerg.Overlord;
import smartlistadapter.SmartList;

import java.util.ArrayList;
import java.util.Stack;

/**
 * Contains the common code used in most activities to show global menus.
 */
public class Activity_options {
	private static final int SETTINGS_REQUEST_CODE = 11235;
	public static final int COMMENT_ACTION = 1;
	public static final int VOTE_ACTION = 2;
	public static final int ARGUMENT_ACTION = 3;
	public static final int ANSWER_ACTION = 4;

	/**
	 * The user pressed the menu button, creates the global common menu.
	 * Pass whether we allow the * refresh option or hide it.
	 *
	 * @return Always returns true.
	 */

	public static boolean onCreateMainOptionsMenu(Activity activity, Menu menu,
												  Overlord overlord) {
		MenuInflater inflater = activity.getMenuInflater();
		inflater.inflate(R.menu.main_activity_menu, menu);
		MenuItem refresh = menu.findItem(R.id.main_refresh);
		if (null != refresh)
			refresh.setVisible(overlord.allow_manual_reload);
		I18n.translate(menu);
		return true;
	}

	public static boolean onCreateChildOptionsMenu(Activity activity, Menu menu,
												   Overlord overlord) {
		MenuInflater inflater = activity.getMenuInflater();
		inflater.inflate(R.menu.child_activity_menu, menu);
		MenuItem refresh = menu.findItem(R.id.child_refresh);
		if (null != refresh)
			refresh.setVisible(overlord.allow_manual_reload);
		MenuItem add = menu.findItem(R.id.add);
		if (null != add)
			add.setVisible(overlord.last_selected_item.can_create_content());
		MenuItem newSearch = menu.findItem(R.id.search);
		if (null != newSearch)
			//if overlord is not searchable or device has no connectivity hide search option
			newSearch.setVisible(Irekia.main_Overlord != null
				&& Irekia.main_Overlord.last_selected_item != null && Irekia.main_Overlord.last_selected_item.is_searchable() && Irekia.isNetworkAvailable());
		I18n.translate(menu);
		return true;
	}

	/**
	 * The user pressed the menu button, creates the global common menu.
	 * Pass whether we allow the * refresh option or hide it.
	 *
	 * @return Always returns true.
	 */
	public static boolean onCreateVolatileOptionsMenu(Activity activity, Menu menu, boolean isSearch) {
		MenuInflater inflater = activity.getMenuInflater();
		inflater.inflate(R.menu.volatile_activity_menu, menu);
		MenuItem new_search = menu.findItem(R.id.new_search);
		if (null != new_search)
			new_search.setVisible(isSearch);
		I18n.translate(menu);
		return true;
	}

	/**
	 * Handles the selection of a menu option, which usually opens an activity.
	 * Pass the overlord to let the menu actions force a data refresh.
	 *
	 * @return Always returns true.
	 */
	public static boolean onOptionsItemSelected(Activity activity, MenuItem item,
												Overlord overlord) {
		Intent intent;
		switch (item.getItemId()) {
			case R.id.main_refresh:
				if (null != overlord)
					overlord.fetch_data();
				break;
			case R.id.child_refresh:
				if (null != overlord && null != overlord.last_selected_item)
					overlord.fetch_child_items(overlord.last_selected_item.get_langcode_url());
				break;
			case R.id.new_search:
				overlord.previous_volatile_items = new Stack<PaginatedResult>();
				overlord.current_volatile_paginated_result = null;
				overlord.volatile_items = new SmartList<Content_item>();
				overlord.volatile_sections = new ArrayList<Section_state>();
				intent = new Intent(activity, Child_list_activity.class);
				intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
				Irekia.must_focus_search = true;
				activity.startActivity(intent);
				break;
			case R.id.search:
				if (activity instanceof Child_list_activity)
					((Child_list_activity) activity).toggle_search();
				break;
			case R.id.add:
				new StepController(Irekia.main_Overlord.last_selected_item.creation_sequence_url, activity).start();
				break;
			case R.id.about:
				intent = new Intent(activity, About_activity.class);
				activity.startActivity(intent);
				break;
			case R.id.settings:
				intent = new Intent(activity, Settings_activity.class);
				activity.startActivityForResult(intent, SETTINGS_REQUEST_CODE);
				break;
			default:
				Toast.makeText(activity, "Selected " + item.getItemId(),
					Toast.LENGTH_SHORT).show();
		}
		return true;
	}


	/**
	 * Method that gets called when the settings activity has finished.
	 * Now we can pick up the list of changes and see if we need to refresh
	 * ourselves.
	 */
	public static void onActivityResult(Activity activity, int requestCode) {
		if (SETTINGS_REQUEST_CODE == requestCode) {
			if (Irekia.update_shared_preferences(activity, true)) {
				// Force clearing the overlords to make Tab_activity reload all.
				Irekia.main_Overlord = null;
				Intent intent = new Intent(activity, Restarter_activity.class);
				activity.startActivity(intent);
				activity.finish();
			}
		}
	}

	public static void doSocialAction(Activity activity, int which, Content_item item) {
		if (!(activity instanceof Social_Activity))
			return;
		if (!Irekia.isNetworkAvailable())
			((Social_Activity) activity).showToast(I18n.e(I18n.Embedded_string.CONNECTION_ERROR));
		else if (Irekia.user_auth_info == null || !Irekia.user_auth_info.is_Logged()) {
			Intent i = new Intent(activity, Login_web_activity.class);
			i.putExtra(Irekia.MAIN_LOGIN_ORIGIN_PARAM, false);
			activity.startActivity(i);
		} else
			switch (which) {
				case COMMENT_ACTION:
					showCommentDialog(activity, item);
					break;
				case ARGUMENT_ACTION:
					showArgumentDialogStep1(activity, item);
					break;
				case VOTE_ACTION:
					showVoteDialog(activity, item);
					break;
				case ANSWER_ACTION:
					showAnswerDialog(activity, item);
					break;
			}
	}

	public static AlertDialog showCommentDialog(final Activity activity, final Content_item item) {
		final AlertDialog alertDialog = new AlertDialog.Builder(activity).create();
		alertDialog.setCancelable(true);
		final EditText input = new EditText(activity);
		input.addTextChangedListener(new TextWatcher() {
			@Override
			public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void afterTextChanged(Editable editable) {
				if (Irekia.main_Overlord.last_selected_item.post_comment_char_limit > 0)
					alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_comment_line_1).equals("")
						? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_comment_line_1)
						.replace("%@", Irekia.user_auth_info.display_name)) + " (" + Irekia.main_Overlord.last_selected_item.post_comment_char_limit + ")");
				if (input.getText().toString().trim().length() == 0)
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
				else
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);
			}
		});
		input.setPadding(10, 10, 10, 10);
		input.setGravity(Gravity.TOP);
		LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
		params.setMargins(15, 15, 15, 15);
		input.setLayoutParams(params);
		input.setLines(4);
		LinearLayout layout = new LinearLayout(activity);
		layout.addView(input);
		if (Irekia.main_Overlord.last_selected_item.post_comment_char_limit > 0)
			input.setFilters(new InputFilter[]{new InputFilter.LengthFilter(Irekia.main_Overlord.last_selected_item.post_comment_char_limit)});
		alertDialog.setView(layout);
		alertDialog.setButton(AlertDialog.BUTTON_POSITIVE, I18n.e(I18n.Embedded_string.SEND), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				String value = input.getText().toString();
				((Social_Activity) activity).showDialog(I18n.e(I18n.Embedded_string.CONNECTING_TO_SERVER));
				new CommentTask(Irekia.main_Overlord.last_selected_item.post_comment_url, value,
					item.id, Irekia.main_Overlord.last_selected_item.unique_id, (Social_Activity) activity).execute();
			}
		});
		alertDialog.setTitle(I18n.p(Irekia.main_Overlord.last_selected_item.post_comment_title));
		if (Irekia.main_Overlord.last_selected_item.post_comment_char_limit > 0)
			alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_comment_line_1).equals("") ? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_comment_line_1).replace("%@", Irekia.user_auth_info.display_name))
				+ " (" + Irekia.main_Overlord.last_selected_item.post_comment_char_limit + ")");
		else
			alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_comment_line_1).equals("") ? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_comment_line_1).replace("%@", Irekia.user_auth_info.display_name)));
		alertDialog.setIcon(R.drawable.ic_menu_new_comment);
		alertDialog.show();
		alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
		return alertDialog;
	}

	public static AlertDialog showArgumentDialogStep1(final Activity activity, final Content_item item) {
		AlertDialog.Builder builder = new AlertDialog.Builder(activity);
		builder.setCancelable(true);
		builder.setPositiveButton(I18n.p(Irekia.main_Overlord.last_selected_item.argument_choice_1), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				if (Irekia.user_auth_info != null && Irekia.user_auth_info.is_Logged())
					showArgumentDialogStep2(activity, item, 1);
				else {
					Intent i = new Intent(activity, Login_web_activity.class);
					i.putExtra(Irekia.MAIN_LOGIN_ORIGIN_PARAM, true);
					activity.startActivity(i);
				}
			}
		});
		builder.setNegativeButton(I18n.p(Irekia.main_Overlord.last_selected_item.argument_choice_2), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				if (Irekia.user_auth_info != null && Irekia.user_auth_info.is_Logged())
					showArgumentDialogStep2(activity, item, 2);
				else {
					Intent i = new Intent(activity, Login_web_activity.class);
					i.putExtra(Irekia.MAIN_LOGIN_ORIGIN_PARAM, true);
					activity.startActivity(i);
				}
			}
		});
		builder.setTitle(I18n.e(I18n.Embedded_string.ARGUMENT));
		builder.setMessage(I18n.p(Irekia.main_Overlord.last_selected_item.argument_choice_title));
		builder.setIcon(R.drawable.ic_menu_new_argument);
		AlertDialog alertDialog = builder.create();
		alertDialog.show();
		return alertDialog;
	}

	public static AlertDialog showArgumentDialogStep2(final Activity activity, final Content_item item, final int choice) {
		final AlertDialog alertDialog = new AlertDialog.Builder(activity).create();
		alertDialog.setCancelable(true);
		final EditText input = new EditText(activity);
		input.addTextChangedListener(new TextWatcher() {
			@Override
			public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void afterTextChanged(Editable editable) {
				if (Irekia.main_Overlord.last_selected_item.post_argument_char_limit > 0)
					alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_argument_line_1).equals("")
						? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_argument_line_1)
						.replace("%@", Irekia.user_auth_info.display_name)) + " (" + String.valueOf(Irekia.main_Overlord.last_selected_item.post_argument_char_limit - input.getText().length()) + ")");
				if (input.getText().toString().trim().length() == 0) {
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);

				} else {
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);
				}
			}
		});
		input.setPadding(10, 10, 10, 10);
		input.setGravity(Gravity.TOP);
		LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
		params.setMargins(15, 15, 15, 15);
		input.setLayoutParams(params);
		input.setLines(4);
		LinearLayout layout = new LinearLayout(activity);
		layout.addView(input);
		if (Irekia.main_Overlord.last_selected_item.post_argument_char_limit > 0)
			input.setFilters(new InputFilter[]{new InputFilter.LengthFilter(Irekia.main_Overlord.last_selected_item.post_argument_char_limit)});
		alertDialog.setView(layout);
		alertDialog.setButton(AlertDialog.BUTTON_POSITIVE, I18n.e(I18n.Embedded_string.SEND), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				String value = input.getText().toString();
				((Social_Activity) activity).showDialog(I18n.e(I18n.Embedded_string.CONNECTING_TO_SERVER));
				new ArgumentTask(Irekia.main_Overlord.last_selected_item.post_argument_url, value,
					choice, item.id, Irekia.main_Overlord.last_selected_item.unique_id, (Social_Activity) activity).execute();
			}
		});
		alertDialog.setTitle(I18n.p(Irekia.main_Overlord.last_selected_item.post_argument_title));

		if (Irekia.main_Overlord.last_selected_item.post_argument_char_limit > 0)
			alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_argument_line_1).equals("")
				? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_argument_line_1)
				.replace("%@", Irekia.user_auth_info.display_name)) + " (" + String.valueOf(Irekia.main_Overlord.last_selected_item.post_argument_char_limit) + ")");
		else
			alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_argument_line_1).equals("")
				? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_argument_line_1)
				.replace("%@", Irekia.user_auth_info.display_name)));
		alertDialog.setIcon(R.drawable.ic_menu_new_argument);
		alertDialog.show();
		alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
		return alertDialog;
	}


	private static AlertDialog showVoteDialog(final Activity activity, final Content_item item) {
		AlertDialog.Builder builder = new AlertDialog.Builder(activity);
		builder.setCancelable(true);
		builder.setPositiveButton(I18n.p(Irekia.main_Overlord.last_selected_item.vote_choice_1), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				((Social_Activity) activity).showDialog(I18n.e(I18n.Embedded_string.CONNECTING_TO_SERVER));
				new VoteTask(Irekia.main_Overlord.last_selected_item.post_vote_url, 1,
					item.id, Irekia.main_Overlord.last_selected_item.unique_id, (Social_Activity) activity).execute();
			}
		});
		builder.setNegativeButton(I18n.p(Irekia.main_Overlord.last_selected_item.vote_choice_2), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				((Social_Activity) activity).showDialog(I18n.e(I18n.Embedded_string.CONNECTING_TO_SERVER));
				new VoteTask(Irekia.main_Overlord.last_selected_item.post_vote_url, 2,
					item.id, Irekia.main_Overlord.last_selected_item.unique_id, (Social_Activity) activity).execute();
			}
		});
		builder.setTitle(I18n.e(I18n.Embedded_string.VOTE));
		builder.setMessage(I18n.p(Irekia.main_Overlord.last_selected_item.vote_choice_title));
		builder.setIcon(R.drawable.ic_menu_vote);
		AlertDialog alertDialog = builder.create();
		alertDialog.show();
		return alertDialog;
	}

	public static AlertDialog showAnswerDialog(final Activity activity, final Content_item item) {
		final AlertDialog alertDialog = new AlertDialog.Builder(activity).create();
		alertDialog.setCancelable(true);
		final EditText input = new EditText(activity);
		input.addTextChangedListener(new TextWatcher() {
			@Override
			public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void afterTextChanged(Editable editable) {
				if (Irekia.main_Overlord.last_selected_item.post_answer_char_limit > 0)
					alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_answer_char_limit).equals("")
						? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_answer_line_1)
						.replace("%@", Irekia.user_auth_info.display_name)) + " (" + Irekia.main_Overlord.last_selected_item.post_answer_char_limit + ")");
				if (input.getText().toString().trim().length() == 0)
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
				else
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);
			}
		});
		input.setPadding(10, 10, 10, 10);
		input.setGravity(Gravity.TOP);
		LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
		params.setMargins(15, 15, 15, 15);
		input.setLayoutParams(params);
		input.setLines(4);
		LinearLayout layout = new LinearLayout(activity);
		layout.addView(input);
		if (Irekia.main_Overlord.last_selected_item.post_answer_char_limit > 0)
			input.setFilters(new InputFilter[]{new InputFilter.LengthFilter(Irekia.main_Overlord.last_selected_item.post_answer_char_limit)});
		alertDialog.setView(layout);
		alertDialog.setButton(AlertDialog.BUTTON_POSITIVE, I18n.e(I18n.Embedded_string.SEND), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				String value = input.getText().toString();
				((Social_Activity) activity).showDialog(I18n.e(I18n.Embedded_string.CONNECTING_TO_SERVER));
				new AnswerTask(Irekia.main_Overlord.last_selected_item.post_answer_url, value,
					item.id, Irekia.main_Overlord.last_selected_item.unique_id, (Social_Activity) activity).execute();
			}
		});
		alertDialog.setTitle(I18n.p(Irekia.main_Overlord.last_selected_item.post_answer_title));
		if (Irekia.main_Overlord.last_selected_item.post_answer_char_limit > 0)
			alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_answer_line_1).equals("") ? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_answer_line_1).replace("%@", Irekia.user_auth_info.display_name))
				+ " (" + Irekia.main_Overlord.last_selected_item.post_answer_char_limit + ")");
		else
			alertDialog.setMessage((I18n.p(Irekia.main_Overlord.last_selected_item.post_answer_line_1).equals("") ? Irekia.user_auth_info.display_name : I18n.p(Irekia.main_Overlord.last_selected_item.post_answer_line_1).replace("%@", Irekia.user_auth_info.display_name)));
		alertDialog.setIcon(R.drawable.ic_menu_answer);
		alertDialog.show();
		alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
		return alertDialog;
	}
}
