//
// OpenIrekia v2.0 Cliente Android
//
// Copyright 2009-2011 eFaber, S.L.
// Copyright 2009-2011 Ejie, S.A.
// Copyrigth 2009-2011 Dirección de Gobierno Abierto y Comunicación en Internet; 
//    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
//    Gobierno Vasco – Eusko Jaurlaritza 
// Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
// por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
// Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
// copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
// Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
// el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
// SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
// Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
// que establece la Licencia
//
//  http://open.irekia.net, openirekia@efaber.net

package net.efaber.irekia.activities;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.webkit.WebView;
import android.widget.ExpandableListView;
import android.widget.Toast;
import net.efaber.irekia.*;
import net.efaber.irekia.models.Activity_progress;
import net.efaber.irekia.models.News_item;
import net.efaber.irekia.models.Section_state;
import net.efaber.irekia.news.News_adapter;
import net.efaber.irekia.news.News_data;
import net.efaber.irekia.zerg.Overlord;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Observable;
import java.util.Observer;

/**
 * Takes care of the full application.
 * This activity is in charge of creating and managing the active tabs. Usually
 * you will first call the static method to validate a JSON which creates the
 * tabs, to later create the true activittitley.
 */
public class Main_activity extends Irekia_expandable_list_activity
	implements Observer, ExpandableListView.OnChildClickListener, Activity_progress {
	private static final String TAG = "Irekia.Main_activity";
	private Post_rotation r;
	/// Stores the last known progress.
	private int last_progress = 10000;
	private boolean must_fetch = true;

	/**
	 * Called when the activity is first created.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.expandable_list);
		getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.title_refresh);
		setTitle(R.string.app_name);
		Bundle extras = getIntent().getExtras();
		if (extras != null)
			must_fetch = extras.getBoolean(Irekia.MAIN_MUST_FETCH, true);
		if (must_fetch) {
			setCustomProgressBarIndeterminateVisibility(true);
			setCustomProgressBarVisibility(true);
		}
		findViewById(R.id.top_refresh).setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (null != Irekia.main_Overlord) {
					setCustomProgressBarIndeterminateVisibility(true);
					setCustomProgressBarVisibility(true);
					Irekia.main_Overlord.fetch_data();
				}
			}
		});
		// Recover rotation persistant variables.
		r = (Post_rotation) getLastNonConfigurationInstance();
		if (null == r)
			r = new Post_rotation();
		// Construct tabs from the overlords.
		if (null == Irekia.main_Overlord)
			return;
		Irekia.main_Overlord.set_progress_reporter(this);
		News_data news_data = (News_data) Irekia.main_Overlord.parsed_data;
		News_adapter adapter = new News_adapter(this, news_data);
		adapter.set(Irekia.main_Overlord);
		ExpandableListView l = getExpandableListView();
		setListAdapter(adapter);
		l.setId(android.R.id.list);
		l.setChildDivider(getResources().getDrawable(R.color.r_light_gray));
		l.setDivider(getResources().getDrawable(R.color.r_light_gray));
		l.setDividerHeight((int) (Irekia.density + 0.5f));
		l.setGroupIndicator(getResources().getDrawable(R.drawable.empty_expandable));
		l.setOnGroupCollapseListener(adapter);
		l.setOnGroupExpandListener(adapter);
		// Set the empty view for the list when there are no items
		Irekia.set_empty_view(l, this);

		// Configure some looks for the listview.
		l.setBackgroundColor(news_data.back_normal_color);
		l.setDrawingCacheBackgroundColor(news_data.back_normal_color);
		l.setCacheColorHint(news_data.back_normal_color);

		l.setOnChildClickListener(this);
		registerForContextMenu(l);
	}

	/**
	 * Check if we have to die.
	 */
	@Override
	public void onResume() {
		super.onResume();
		final boolean lang_changed = !I18n.get_effective_language()
			.equals(Irekia.last_lang_preference);
		if (Irekia.main_Overlord != null)
			Irekia.main_Overlord.set_progress_reporter(this);
		if (Irekia.needs_restarting || null == Irekia.main_Overlord || lang_changed) {
			Irekia.main_Overlord = null;
			Intent intent = new Intent(this, Startup_activity.class);
			startActivityForResult(intent, Startup_activity.STARTUP_REQUEST_CODE);
		} else {
			Irekia.main_Overlord.items.addObserver(this);
			if (!r.did_fetch && must_fetch) {
				Irekia.main_Overlord.fetch_data();
				r.did_fetch = true;
			} else {
				Irekia.main_Overlord.fetch_if_ttl_expired();
			}
			if (r.last_fetch < Irekia.main_Overlord.last_update)
				update(null, null);
		}
	}

	@Override
	protected void onRestart() {
		super.onRestart();
	}

	public void onPause() {
		super.onPause();
		if (null == Irekia.main_Overlord)
			return;

		Irekia.main_Overlord.items.deleteObserver(this);
		Irekia.main_Overlord.cancel_main_fetching();
	}

	/**
	 * Method that gets called when the startup intent has finished.
	 * We simply force a restart.
	 */
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (Startup_activity.STARTUP_REQUEST_CODE == requestCode) {
			// Did the startup activity succeed in downloading stuff?
			if (null == Irekia.main_Overlord)
				finish();
			else
				restart_app();
		} else
			Activity_options.onActivityResult(this, requestCode);
	}

	/**
	 * Handles the user clicking or selecting an item with.
	 * The item's url will be loaded in a new web activity.
	 */
	@Override
	public boolean onChildClick(ExpandableListView parent, View view,
								int groupPosition, int childPosition, long id) {
		News_item item = (News_item) parent.getExpandableListAdapter().getChild(
			groupPosition, childPosition);
		Irekia.main_Overlord.last_selected_item = item;
		Irekia.main_Overlord.child_sections = new ArrayList<Section_state>();
		Irekia.main_Overlord.child_items.clear();
		if (null != item) {
			item.start_child_activity(this, Irekia.main_Overlord);
			return true;
		} else {
			Toast.makeText(this, "Internal error processing item id!",
				Toast.LENGTH_SHORT).show();
			return false;
		}
	}

	/**
	 * Parses the tab json and creates the controllers.
	 *
	 * @return list of tabs that can be used to create the tab activity, or null if
	 *         there were problems with the parsing.
	 */
	static public Overlord parse_app_data(JSON json) {
		JSON data = json.get_map("root_controller", null);
		ArrayList<Object> tabs = json.get_array("tabs", null, HashMap.class);
		JSON gallery_data = null;
		JSON news_data = null;
		ArrayList<String> tags_regex = null;
		ArrayList<String> videos_regex = null;
		if (null != tabs || tabs.size() > 1) {
			for (Object tab_stub : tabs) {
				JSON tab_json = JSON.map(tab_stub);
				if (tab_json != null && tab_json.get_string("class_type", "").equals("News_view_controller")) {
					news_data = tab_json.get_map("data", null);
					if (tab_json.get_map("data", null) != null) {
						if (tags_regex == null || tags_regex.size() == 0)
							tags_regex = tab_json.get_map("data", null).get_array("tags_regex", new ArrayList<String>(), String.class);
						if (videos_regex == null || videos_regex.size() == 0)
							videos_regex = tab_json.get_map("data", null).get_array("videos_regex", new ArrayList<String>(), String.class);
					}
				} else if (tab_json != null && tab_json.get_string("class_type", "").equals("Gallery_view_controller"))
					gallery_data = tab_json.get_map("data", null);
			}
		}

		JSON misc = json.get_map("misc", null);
		String login_webview_url = "";
		ArrayList<String> intralinks_regex = new ArrayList<String>();
		if (misc != null) {
			login_webview_url = misc.get_string("login_webview_url", "");
			intralinks_regex = misc.get_array("intralinks_regex", new ArrayList<String>(), String.class);
		}
		try {
			Overlord overlord = new Overlord(data, true);
			overlord.generic_gallery_data = gallery_data;
			overlord.tags_regex = tags_regex;
			overlord.videos_regex = videos_regex;
			overlord.generic_news_data = news_data;
			overlord.login_webview_url = login_webview_url;
			overlord.intralinks_regex = intralinks_regex;
			return overlord;
		} catch (ParseException e) {
			Log.w(TAG, "Couldn't parse overlord. " + e.getLocalizedMessage());
			return null;
		}
	}

	/**
	 * Force finishing this app while also launching the restart handler.
	 */
	public void restart_app() {
		Intent intent = new Intent(this, Restarter_activity.class);
		startActivity(intent);
		finish();
	}

	/**
	 * Pass some parameters to our future self after rotation.
	 * We want to preserve some data like the r.last_fetch timestamp after a
	 * rotation, so we communicate it here.
	 */
	@Override
	public Object onRetainNonConfigurationInstance() {
		return r;
	}

	@Override
	public void set_progress(int progress) {
		last_progress = progress;
		setCustomProgress(progress);
	}

	@Override
	public void update(Observable observable, Object data) {
		//Update the adapter to refresh items
		News_adapter adapter = (News_adapter) getExpandableListAdapter();
		adapter.set(Irekia.main_Overlord);
		r.last_fetch = Irekia.main_Overlord.last_update;
		update_collapsed_state(getExpandableListView(), adapter);
	}

	/**
	 * Modifies the collapsed state of all groups in a listview.
	 * The collapsed state comes out of the attribute for the Section_state objects.
	 */
	protected void update_collapsed_state(ExpandableListView list,
										  News_adapter adapter) {
		for (int f = 0; f < adapter.getGroupCount(); f++) {
			Section_state section = adapter.getGroup(f);
			if (null != section) {
				if (section.collapsed)
					list.collapseGroup(f);
				else
					list.expandGroup(f);
			} else {
				list.expandGroup(f);
			}
		}
	}

	/**
	 * Custom private class to preserve data between rotations.
	 */
	private class Post_rotation {
		public long last_fetch;
		public boolean did_fetch;
	}

/*********************** Common activity options menu ***********************/

	/**
	 * The user pressed the menu button, create the menu.
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		return Activity_options.onCreateMainOptionsMenu(this, menu, Irekia.main_Overlord);
	}

	/**
	 * The user selected a menu option, what should we do?
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		return Activity_options.onOptionsItemSelected(this, item, Irekia.main_Overlord);
	}

}
