package net.efaber.irekia.cache;

import android.util.Log;
import net.efaber.irekia.Irekia;
import net.efaber.irekia.JSON;
import org.codehaus.jackson.JsonGenerationException;
import org.codehaus.jackson.map.JsonMappingException;

import java.io.*;

/**
 * Class in charge of dealing with Overlods´ cached json, save, load, delete...
 */
public final class Cache_Manager {
	private static final String TAG = "Irekia.Cache_Manager";

	/**
	 * Check if overlod.json is cached or not
	 */
	public static boolean exist_json(String id) {
		File file = new java.io.File(Irekia.cache_dir, id + ".json");
		return file.exists();
	}


	/**
	 * Retrieve cached overlod.json data
	 */
	public static JSON load_json(String id) {
		final String name = Irekia.cache_dir + "/" + id + ".json";
		FileInputStream input = null;
		try {
			input = new FileInputStream(name);
		} catch (FileNotFoundException e) {
			input = null;
		}
		JSON json = null;
		try {
			json = new JSON(input);
		} catch (JsonGenerationException e) {
			Log.w(TAG, "Can't load " + name + ".\n" + e.getLocalizedMessage());
			e.printStackTrace();
		} catch (JsonMappingException e) {
			Log.w(TAG, "Can't load " + name + ".\n" + e.getLocalizedMessage());
			e.printStackTrace();
		} catch (IOException e) {
			Log.w(TAG, "Can't load " + name + ".\n" + e.getLocalizedMessage());
			e.printStackTrace();
		} finally {
			if (input != null) try {
				input.close();
			} catch (IOException e) {
				Log.w(TAG, "Can't close " + name + ".\n" + e.getLocalizedMessage());
			}
		}
		return json;
	}

	/**
	 * Save overlord data to overlod.json cache
	 */
	public static void save_json(String id, String json) {
		final String name = Irekia.cache_dir + "/" + id + ".json";
		FileOutputStream output;
		try {
			output = new FileOutputStream(name);
		} catch (FileNotFoundException e) {
			Log.w(TAG, "Can't save " + name + ".\n" + e.getLocalizedMessage());
			e.printStackTrace();
			return;
		}
		try {
			output.write(json.getBytes());
		} catch (JsonGenerationException e) {
			Log.w(TAG, "Can't save " + name + ".\n" + e.getLocalizedMessage());
			e.printStackTrace();
		} catch (JsonMappingException e) {
			Log.w(TAG, "Can't save " + name + ".\n" + e.getLocalizedMessage());
			e.printStackTrace();
		} catch (IOException e) {
			Log.w(TAG, "Can't save " + name + ".\n" + e.getLocalizedMessage());
			e.printStackTrace();
		} finally {
			if (output != null) try {
				output.close();
			} catch (IOException e) {
				Log.w(TAG, "Can't close " + name + ".\n" + e.getLocalizedMessage());
			}
		}
	}

	/**
	 * Delete cached Overlod.json
	 */
	public static void delete_json(String id) {
		final String name = Irekia.cache_dir + "/" + id + ".json";
		new File(name).delete();
	}

	public static void delete_all_cache_content() {
		File cacheDir = new File(Irekia.cache_dir);
		File[] files = cacheDir.listFiles();
		if (files != null) {
			for (File file : files)
				file.delete();
		}
	}
}
