package net.efaber.irekia.childcontroller;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ActivityNotFoundException;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;
import net.efaber.irekia.I18n;
import net.efaber.irekia.Irekia;
import net.efaber.irekia.login.Login_web_activity;
import net.efaber.irekia.models.Content_item;
import net.efaber.irekia.volatilecontroller.Volatile_gallery_activity;
import net.efaber.irekia.volatilecontroller.Volatile_list_activity;

/**
 * The actual class extending the web client to implement our custom behaviour.
 * We just open the OS browser if available.
 */
public class Web_view_client extends WebViewClient {

	private final Activity activity;
	private Content_item item;
	private final boolean isVolatile;

	public Web_view_client(Activity activity, boolean aVolatile) {
		this.activity = activity;
		isVolatile = aVolatile;
	}

	public void set_item(Content_item item) {
		this.item = item;
	}

	@Override
	public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
		AlertDialog alertDialog = Irekia.createNoConnectionDialog(activity);
		alertDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
			@Override
			public void onDismiss(DialogInterface dialogInterface) {
				activity.finish();
			}
		});
		alertDialog.show();
	}


	@Override
	public boolean shouldOverrideUrlLoading(WebView view, String url) {
		return isVolatile ? checkIncomingUrlVolatile(view, url) : checkIncomingUrl(view, url);
	}

	private boolean checkIncomingUrl(WebView view, String url) {
		if (url.matches("http://www.irekia.euskadi.net/[a-z]*/mob_app/photos[?]?news_id=([0-9]*)")) {
			if (!Irekia.main_Overlord.current_paginated_result.getSearch_url().equals(Irekia.main_Overlord.previous_child_items.peek().getSearch_url()))
				Irekia.main_Overlord.save_previous_items(true);
			Intent intent = new Intent().setClass(activity, Volatile_gallery_activity.class);
			intent.putExtra(Irekia.TITLE_PARAM, I18n.e(I18n.Embedded_string.IREKIA_PHOTO_WINDOW_TITLE) + item.get_title());
			intent.putExtra(Irekia.MODE_PARAM, Irekia.MODE_PHOTO);
			intent.putExtra(Irekia.SEARCH_URL_PARAM, url);
			activity.startActivity(intent);
			return true;
		}
		for (String Vregex : (Irekia.main_Overlord.last_selected_item.videos_regex != null
			&& Irekia.main_Overlord.last_selected_item.videos_regex.size() > 0)
			? Irekia.main_Overlord.last_selected_item.videos_regex : Irekia.main_Overlord.videos_regex)
			if (url.matches(Vregex) || url.matches(Vregex.replaceAll("/mob_app/", "/[a-z]*/mob_app/"))) {
				if (!Irekia.main_Overlord.current_paginated_result.getSearch_url().equals(Irekia.main_Overlord.previous_child_items.peek().getSearch_url()))
					Irekia.main_Overlord.save_previous_items(true);
				Intent intent = new Intent().setClass(activity, Volatile_list_activity.class);
				intent.putExtra(Irekia.MODE_PARAM, Irekia.MODE_VIDEO);
				intent.putExtra(Irekia.TITLE_PARAM, I18n.e(I18n.Embedded_string.IREKIA_VIDEO_WINDOW_TITLE) + item.get_title());
				intent.putExtra(Irekia.SEARCH_URL_PARAM, url);
				activity.startActivity(intent);
				return true;
			}
		for (String Tregex : (Irekia.main_Overlord.last_selected_item.tags_regex != null
			&& Irekia.main_Overlord.last_selected_item.tags_regex.size() > 0)
			? Irekia.main_Overlord.last_selected_item.tags_regex : Irekia.main_Overlord.tags_regex)
			if (url.matches(Tregex) || url.matches(Tregex.replaceAll("/mob_app/", "/[a-z]*/mob_app/"))) {
				if (!Irekia.main_Overlord.current_paginated_result.getSearch_url().equals(Irekia.main_Overlord.previous_child_items.peek().getSearch_url()))
					Irekia.main_Overlord.save_previous_items(true);
				Intent intent = new Intent().setClass(activity, Volatile_list_activity.class);
				intent.putExtra(Irekia.TITLE_PARAM, I18n.e(I18n.Embedded_string.IREKIA_TAGS_WINDOW_TITLE) + getTag(url));
				intent.putExtra(Irekia.MODE_PARAM, Irekia.MODE_TAGS);
				intent.putExtra(Irekia.SEARCH_URL_PARAM, url);
				activity.startActivity(intent);
				return true;
			}
		for (String Tregex : Irekia.main_Overlord.intralinks_regex)
			if (!Irekia.isNetworkAvailable()) {
				Toast.makeText(activity, I18n.e(I18n.Embedded_string.CONNECTION_ERROR), Toast.LENGTH_LONG).show();
				return true;
			} else {
				if (url.equals(Irekia.main_Overlord.login_webview_url.replace("_LNG_", I18n.get_effective_language()))) {
					Intent intent = new Intent(activity, Login_web_activity.class);
					intent.putExtra(Irekia.MAIN_LOGIN_ORIGIN_PARAM, true);
					activity.startActivity(intent);
					return true;
				} else if (url.matches(Tregex) || url.matches(Tregex.replaceAll("/mob_app/", "/[a-z]*/mob_app/"))) {
					view.loadUrl(url);
					return true;
				}
			}
		try {
			Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
			activity.startActivity(intent);
			return true;
		} catch (ActivityNotFoundException e) {
			view.loadUrl(url);
			return true;
		}
	}

	private boolean checkIncomingUrlVolatile(WebView view, String url) {
		if (url.matches("http://www.irekia.euskadi.net/[a-z]*/mob_app/photos[?]?news_id=([0-9]*)")) {
			Irekia.main_Overlord.save_previous_volatiles();
			Intent intent = new Intent().setClass(activity, Volatile_gallery_activity.class);
			intent.putExtra(Irekia.MODE_PARAM, Irekia.MODE_PHOTO);
			intent.putExtra(Irekia.TITLE_PARAM, I18n.e(I18n.Embedded_string.IREKIA_PHOTO_WINDOW_TITLE) + item.get_title());
			intent.putExtra(Irekia.SEARCH_URL_PARAM, url);
			activity.startActivity(intent);
			return true;
		}
		for (String Vregex : (Irekia.main_Overlord.last_selected_item.videos_regex != null
			&& Irekia.main_Overlord.last_selected_item.videos_regex.size() > 0)
			? Irekia.main_Overlord.last_selected_item.videos_regex : Irekia.main_Overlord.videos_regex)
			if (url.matches(Vregex) || url.matches(Vregex.replaceAll("/mob_app/", "/[a-z]*/mob_app/"))) {
				Irekia.main_Overlord.save_previous_volatiles();
				Intent intent = new Intent().setClass(activity, Volatile_list_activity.class);
				intent.putExtra(Irekia.MODE_PARAM, Irekia.MODE_VIDEO);
				intent.putExtra(Irekia.TITLE_PARAM, I18n.e(I18n.Embedded_string.IREKIA_VIDEO_WINDOW_TITLE) + item.get_title());
				intent.putExtra(Irekia.SEARCH_URL_PARAM, url);
				activity.startActivity(intent);
				return true;
			}
		for (String Tregex : (Irekia.main_Overlord.last_selected_item.tags_regex != null
			&& Irekia.main_Overlord.last_selected_item.tags_regex.size() > 0)
			? Irekia.main_Overlord.last_selected_item.tags_regex : Irekia.main_Overlord.tags_regex)
			if (url.matches(Tregex) || url.matches(Tregex.replaceAll("/mob_app/", "/[a-z]*/mob_app/"))) {
				Irekia.main_Overlord.save_previous_volatiles();
				Intent intent = new Intent().setClass(activity, Volatile_list_activity.class);
				intent.putExtra(Irekia.MODE_PARAM, Irekia.MODE_TAGS);
				intent.putExtra(Irekia.TITLE_PARAM, I18n.e(I18n.Embedded_string.IREKIA_TAGS_WINDOW_TITLE) + getTag(url));
				intent.putExtra(Irekia.SEARCH_URL_PARAM, url);
				activity.startActivity(intent);
				return true;
			}
		for (String Tregex : Irekia.main_Overlord.intralinks_regex)
			if (!Irekia.isNetworkAvailable()) {
				Toast.makeText(activity, I18n.e(I18n.Embedded_string.CONNECTION_ERROR), Toast.LENGTH_LONG).show();
				return true;
			} else {
				if (url.equals(Irekia.main_Overlord.login_webview_url.replace("_LNG_", I18n.get_effective_language()))) {
					Intent intent = new Intent(activity, Login_web_activity.class);
					intent.putExtra(Irekia.MAIN_LOGIN_ORIGIN_PARAM, true);
					activity.startActivity(intent);
					return true;
				} else if (url.matches(Tregex) || url.matches(Tregex.replaceAll("/mob_app/", "/[a-z]*/mob_app/"))) {
					view.loadUrl(url);
					return true;
				}
			}
		try {
			Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
			activity.startActivity(intent);
			return true;
		} catch (ActivityNotFoundException e) {
			view.loadUrl(url);
			return true;
		}
	}


	private String getTag(String url) {
		Uri uri = Uri.parse(url);
		return (uri.getQueryParameter("q") != null) ? uri.getQueryParameter("q") : "";
	}

}
