//
// OpenIrekia v2.0 Cliente Android
//
// Copyright 2009-2011 eFaber, S.L.
// Copyright 2009-2011 Ejie, S.A.
// Copyrigth 2009-2011 Dirección de Gobierno Abierto y Comunicación en Internet; 
//    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
//    Gobierno Vasco – Eusko Jaurlaritza 
// Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
// por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
// Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
// copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
// Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
// el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
// SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
// Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
// que establece la Licencia
//
//  http://open.irekia.net, openirekia@efaber.net

package net.efaber.irekia.models;

import android.content.Context;
import net.efaber.irekia.I18n;
import net.efaber.irekia.Irekia;
import net.efaber.irekia.JSON;
import net.efaber.irekia.zerg.Overlord;
import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.annotate.JsonUnwrapped;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import static junit.framework.Assert.assertTrue;

public class Content_item
	implements Sharing_tags {
	/// Identifier of the item.
	public int id;

	/// Unix timestamp.
	public int expiration_date;

	/// Sorting identifier.
	public long sort_id;

	/// Title of the object, can't be null.
	public String title;

	/// Retrieval URL of the data this item points to if available.
	public String url;

	/// Public URL for user consumption if available.
	public String share_url;

	/// URL of the image associated with this item if available.
	public String image;

	/// If yes, the content is never cached, always grabbed from the net.
	public boolean online;

	/// Type of the item, can't be null.
	public String class_type;

	/// Further data for the item if available.
	/// Unwrap the object when desearilizing to avoid having data:data:{
	@JsonUnwrapped
	public JSON data;

	/// List of section identifiers this item is child of if available.
	public ArrayList<Integer> section_ids;

	@JsonIgnore
	public int unique_id = -1;

	/// Watchdog to know if last fetched data has older items
	@JsonIgnore
	public boolean has_older = true;


	/// Maximum number of items to show in the view. Excedent is purged.
	@JsonIgnore
	public int cache_size = 50;

	@JsonIgnore
	public boolean can_fetch_more = false;

	/// And this is url + the current language code if necessary.
	public String search_bar_url;

	/// Public URL for user consumption if available.
	public ArrayList<String> tags_regex = new ArrayList<String>();
	/// Public URL for user consumption if available.
	public ArrayList<String> videos_regex = new ArrayList<String>();

	public String follow_url;
	public String unfollow_url;
	public String follow_data;
	public boolean accepts_comments;
	public boolean accepts_votes;
	public boolean accepts_arguments;
	public boolean accepts_answers;
	public String creation_sequence_url;
	public int empty_table_message;
	public JSON vote_params;
	public int vote_choice_title;
	public int vote_choice_1;
	public int vote_choice_2;
	public String post_vote_url;
	public int post_vote_line_2;
	public JSON comment_params;
	public int post_comment_title;
	public int post_comment_char_limit;
	public int post_comment_line_1;
	public int post_comment_line_2;
	public String post_comment_url;
	public JSON add_argument_params;
	public int post_argument_title;
	public int post_argument_char_limit;
	public int post_argument_line_1;
	public int argument_choice_title;
	public int argument_choice_1;
	public int argument_choice_2;
	public String post_argument_url;
	public JSON answer_params;
	public String post_answer_url;
	public int post_answer_title;
	public int post_answer_line_1;
	public int post_answer_line_2;
	public int post_answer_char_limit;
	@JsonIgnore
	public boolean isFollowing = false;
	public boolean new_window = false;

	/**
	 * Creates a Content_item element from a JSON dictionary.
	 */
	public Content_item(JSON json) throws ParseException {
		id = json.get_int("id", -1);
		sort_id = json.get_long("sort_id", id);
		title = json.get_string("title", null);
		url = json.get_url("url", null);
		share_url = json.get_url("share_url", null);
		image = json.get_url("image", null);
		online = json.get_bool("online", false);
		class_type = json.get_string("class_type", null);
		data = json.get_map("data", null);
		section_ids = json.get_array("section_ids", null, Integer.class);
		expiration_date = json.get_int("expiration_date", -1);
		accepts_comments = json.get_bool("accepts_comments", false);
		accepts_votes = json.get_bool("accepts_votes", false);
		accepts_arguments = json.get_bool("accepts_arguments", false);
		accepts_answers = json.get_bool("accepts_answers", false);
		new_window = json.get_bool("new_window", true);

		if (data != null) {
			unique_id = data.get_int("unique_id", -1);
			cache_size = data.get_int("cache_size", 50);
			videos_regex = data.get_array("videos_regex", new ArrayList<String>(), String.class);
			tags_regex = data.get_array("tags_regex", new ArrayList<String>(), String.class);
			search_bar_url = data.get_string("search_bar", "");
			follow_url = data.get_string("follow_url", "");
			unfollow_url = data.get_string("unfollow_url", "");
			follow_data = data.get_string("follow_data", "");
			empty_table_message = data.get_int("empty_table_message", -1);
			creation_sequence_url = data.get_string("creation_sequence_url", "");
			vote_params = data.get_map("vote_params", null);
			if (vote_params != null) {
				ArrayList<Integer> post_vote_choice = vote_params.get_array("post_vote_choice", new ArrayList<Integer>(), Integer.class);
				vote_choice_title = post_vote_choice.get(0);
				vote_choice_1 = post_vote_choice.get(1);
				vote_choice_2 = post_vote_choice.get(2);
				post_vote_line_2 = vote_params.get_int("post_vote_line_2", -1);
				post_vote_url = vote_params.get_string("post_vote_url", "");
			}
			comment_params = data.get_map("comment_params", null);
			if (comment_params != null) {
				post_comment_title = comment_params.get_int("post_comment_title", -1);
				post_comment_char_limit = comment_params.get_int("post_comment_char_limit", 0);
				post_comment_line_1 = comment_params.get_int("post_comment_line_1", -1);
				post_comment_line_2 = comment_params.get_int("post_comment_line_2", -1);
				post_comment_url = comment_params.get_string("post_comment_url", "");
			}
			add_argument_params = data.get_map("add_argument_params", null);
			if (add_argument_params != null) {
				post_argument_title = add_argument_params.get_int("post_argument_title", -1);
				ArrayList<Integer> post_argument_choice = add_argument_params.get_array("post_argument_choice", new ArrayList<Integer>(), Integer.class);
				argument_choice_title = post_argument_choice.get(0);
				argument_choice_1 = post_argument_choice.get(1);
				argument_choice_2 = post_argument_choice.get(2);
				post_argument_char_limit = add_argument_params.get_int("post_argument_char_limit", 0);
				post_argument_line_1 = add_argument_params.get_int("post_argument_line_1", -1);
				post_argument_url = add_argument_params.get_string("post_argument_url", "");
			}
			answer_params = data.get_map("answer_params", null);
			if (answer_params != null) {
				post_answer_url = answer_params.get_string("post_answer_url", "");
				post_answer_title = answer_params.get_int("post_answer_title", -1);
				post_answer_line_1 = answer_params.get_int("post_answer_line_1", -1);
				post_answer_line_2 = answer_params.get_int("post_answer_line_2", -1);
				post_answer_char_limit = answer_params.get_int("post_answer_char_limit", 0);
			}
		}

		if (null == title || id < 1)
			throw new ParseException("Null title or invalid id", 0);

		// Verify that the type is one of the allowed types. */
		if (null == class_type) {
			class_type = default_controller();
		} else {
			boolean valid = default_controller().equals(class_type);

			if (!valid) {
				// TODO: Perform class instantiation from string name.
			}
		}
	}

	public String get_empty_message() {
		return empty_table_message != -1 ? I18n.p(empty_table_message) : I18n.e(I18n.Embedded_string.IREKIA_NO_RESULTS);
	}


	/**
	 * Returns the specific class name for the class.
	 * Use this to let the Content_item class know the name of the specific
	 * controller for the class inheriting from FLContent_item. This
	 * should be the name of the default controller when the application
	 * data protocol doesn't specify any.
	 */
	public String default_controller() {
		return "No controller";
	}

	/**
	 * Generates the JSON like map ready to be serialized.
	 */
	/// Ignore the getter when desearilizing to avoid having an _json obj
	@JsonIgnore
	public JSON get_json() {
		HashMap<String, Object> json = new HashMap<String, Object>(18);
		json.put("id", id);
		json.put("sort_id", (long) sort_id);
		json.put("title", title);
		if (null != url) json.put("url", url);
		if (null != share_url) json.put("share_url", share_url);
		if (null != image) json.put("image", image);
		if (online) json.put("online", online);
		if (null != class_type) json.put("class_type", class_type);
		if (null != data) json.put("data", data);
		if (null != section_ids && section_ids.size() > 0)
			json.put("section_ids", section_ids);
		if (expiration_date > 0) json.put("expiration_date", expiration_date);

		return new JSON(json);
	}

	/**
	 * String that represents the object. Helpful for debugging.
	 */
	public String toString() {
		return "Content {id:" + id + ", sort_id:" + sort_id + ", title:'" +
			title + "', url:'" + url + "', share_url:'" + share_url + "', image:'" +
			image + "', data:" + data + "}";
	}

	/**
	 * Call this to open an activity with the details of the item.
	 * This method doesn't work at the Content_item level, it has to be overriden
	 * by subclasses, which should not call this version (just asserts).
	 */
	public void start_activity(Context context, final Overlord overlord) {
		assertTrue("Don't call this at Content_item level", false);
	}

/**************** Sharing_tags interface implementation **********************/

	/**
	 * Returns the title of the item. See @Sharing_tags for more info.
	 */
	/// Ignore the getter when desearilizing to avoid having an _title obj
	@JsonIgnore
	public String get_title() {
		if (null != title)
			return title;
		else
			return "";
	}

	/**
	 * Returns the photo description of the item. See @Sharing_tags for more info.
	 * Since Content_item objects don't know anything about photos, this method
	 * should be overriden by a child class.
	 */
	/// Ignore the getter when desearilizing to avoid having an _photo_desc obj
	@JsonIgnore
	public String get_photo_desc() {
		return "";
	}

	@JsonIgnore
	public String get_langcode_url() {
		return url != null ? url.replace("_LNG_", I18n.current_langcode) : "";
	}

	/**
	 * Returns the public url of the item. See @Sharing_tags for more info.
	 */
	/// Ignore the getter when desearilizing to avoid having an _url obj
	@JsonIgnore
	public String get_url() {
		if (null != share_url)
			return share_url;
		else
			return "";
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (!(o instanceof Content_item)) return false;

		Content_item that = (Content_item) o;

		if (id != that.id) return false;
		if (url != null ? !url.equals(that.url) : that.url != null) return false;

		return true;
	}

	@Override
	public int hashCode() {
		int result = id;
		result = 31 * result + (url != null ? url.hashCode() : 0);
		return result;
	}

	@JsonIgnore
	public boolean isOverlord() {
		return unique_id != -1;
	}

	@JsonIgnore
	public String getUniqueUrl() {
		if (url == null)
			return "generic_json";
		String[] split = url.split("/");
		if (split.length > 0)
			return split[split.length - 1];
		return "generic_json";
	}

	public boolean is_searchable() {
		return search_bar_url != null && !search_bar_url.equals("");
	}

	@JsonIgnore
	public boolean can_fetch_more() {
		if (!Irekia.isNetworkAvailable())
			return false;
		return isOverlord() ? (url.contains("news") ||
			url.contains("photos") ||
			url.contains("videos")) : true;
	}

	@JsonIgnore
	public boolean can_create_content() {
		return isOverlord() && creation_sequence_url != null && !creation_sequence_url.equals("");
	}

}
