//
// OpenIrekia v2.0 Cliente Android
//
// Copyright 2009-2011 eFaber, S.L.
// Copyright 2009-2011 Ejie, S.A.
// Copyrigth 2009-2011 Dirección de Gobierno Abierto y Comunicación en Internet; 
//    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
//    Gobierno Vasco – Eusko Jaurlaritza 
// Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
// por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
// Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
// copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
// Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
// el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
// SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
// Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
// que establece la Licencia
//
//  http://open.irekia.net, openirekia@efaber.net

package net.efaber.irekia.models;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import android.widget.Toast;
import net.efaber.irekia.I18n;
import net.efaber.irekia.Irekia;
import net.efaber.irekia.JSON;
import net.efaber.irekia.activities.Workaround_Activity;
import net.efaber.irekia.childcontroller.Child_gallery_activity;
import net.efaber.irekia.childcontroller.Child_video_facade_activity;
import net.efaber.irekia.childcontroller.Child_web_activity;
import net.efaber.irekia.follow.FollowTask;
import net.efaber.irekia.login.Login_web_activity;
import net.efaber.irekia.login.LogoutTask;
import net.efaber.irekia.unfollow.UnfollowTask;
import net.efaber.irekia.volatilecontroller.Volatile_gallery_activity;
import net.efaber.irekia.volatilecontroller.Volatile_video_facade_activity;
import net.efaber.irekia.volatilecontroller.Volatile_web_activity;
import net.efaber.irekia.zerg.Overlord;

import java.text.ParseException;

public class News_item extends Content_item {
	private static final String TAG = "Irekia.News_item";

	/// String with the contents of the news, can't be null.
	public String body;

	/// Text for the footer, if available.
	public String footer;


	/**
	 * Creates a News_item element from a JSON dictionary.
	 */
	public News_item(JSON json) throws ParseException {
		super(json);
		body = json.get_string("body", null);
		footer = json.get_string("footer", null);

		if (null == body)
			throw new ParseException("Null body for news item", 0);
	}

	/**
	 * Returns the controller class name for a news item.
	 * See parent definition for more info.
	 */
	public String default_controller() {
		return "News_view_controller";
	}

	/**
	 * Generates the JSON like map ready to be serialized.
	 */
	public JSON get_json() {
		JSON json = super.get_json();
		if (null == json)
			return null;
		if (null != body && !"".equals(body)) json.data.put("body", body);
		if (null != footer) json.data.put("footer", footer);

		return json;
	}

	/**
	 * String that represents the object. Helpful for debugging.
	 */
	public String toString() {
		final String micro_body = (null == body) ? null :
			(body.length() < 21 ? body :
				body.substring(0, Math.min(20, body.length())) + "...");
		return "News_item {body:'" + micro_body + "', " + super.toString() + "}";
	}

	/**
	 * Call this to open a Web_activity to read the item.
	 */
	public void start_child_activity(Activity activity, final Overlord overlord) {
		Intent intent = null;
		if (!class_type.equals("Login_cell") && !class_type.equals("Follow_cell") && (url == null || url.equals("")))
			return;
		if (class_type.equals("Login_cell")) {
			if (checkConnectedAndLogged(activity, true)) {
				activity.setProgress(1000);
				new LogoutTask().setActivity(activity).execute(Irekia.main_Overlord.auth_info_url.replace("_LNG_", I18n.get_effective_language()));
			} else
				return;

		} else if (class_type.equals("Follow_cell")) {
			if (checkConnectedAndLogged(activity, false)) {
				if (isFollowing) {
					activity.setProgress(1000);
					new UnfollowTask(activity).execute(unfollow_url.replace("_LNG_", I18n.get_effective_language()));
				} else if (!isFollowing) {
					activity.setProgress(1000);
					new FollowTask(activity).execute(follow_url.replace("_LNG_", I18n.get_effective_language()), follow_data);
				}
			} else
				return;
		} else if (unique_id != -1 && class_type.equals("News_view_controller"))
			intent = new Intent(activity, Workaround_Activity.class);
		else if (class_type.equals(default_controller()) && unique_id == -1) {
			intent = new Intent(activity, Child_web_activity.class);
		} else if (class_type.equals("Gallery_view_controller")) {
			intent = new Intent(activity, Child_gallery_activity.class);
		} else if (null != data &&
			data.get_url("preview_url", "").length() > 0) {
			intent = new Intent(activity, Child_video_facade_activity.class);
		}
		if (null != intent) {
			intent.putExtra(Irekia.CONTENT_ID_PARAM, id);
			intent.putExtra(Irekia.TITLE_PARAM, title);
			activity.startActivity(intent);
		} else {
			Log.w(TAG, "Don't know how to deal with item (" + overlord.id +
				"." + id + ")");
		}
	}

	private boolean checkConnectedAndLogged(Activity activity, boolean origin) {
		if (!Irekia.isNetworkAvailable()) {
			Toast.makeText(activity, I18n.e(I18n.Embedded_string.CONNECTION_ERROR), Toast.LENGTH_LONG).show();
			return false;
		}
		if (!Irekia.user_auth_info.is_Logged()) {
			Intent intent = new Intent(activity, Login_web_activity.class);
			intent.putExtra(Irekia.MAIN_LOGIN_ORIGIN_PARAM, origin);
			intent.putExtra(Irekia.CONTENT_ID_PARAM, id);
			intent.putExtra(Irekia.TITLE_PARAM, title);
			activity.startActivity(intent);
			return false;
		}
		return true;
	}


	/**
	 * Call this to open a Web_activity to read the item.
	 */

	public void start_volatile_activity(Context context, final Overlord overlord, int lastMode) {
		Intent intent = null;
		if (null != data &&
			data.get_url("preview_url", "").length() > 0) {
			intent = new Intent(context, Volatile_video_facade_activity.class);
		} else if (class_type.equals("Gallery_view_controller")) {
			intent = new Intent(context, Volatile_gallery_activity.class);
		} else {
			intent = new Intent(context, Volatile_web_activity.class);
		}
		intent.putExtra(Irekia.CONTENT_ID_PARAM, id);
		intent.putExtra(Irekia.MODE_PARAM, lastMode);
		intent.putExtra(Irekia.TITLE_PARAM, title);
		intent.putExtra(Irekia.SEARCH_URL_PARAM, url);
		context.startActivity(intent);
	}
}
