//
// OpenIrekia v2.0 Cliente Android
//
// Copyright 2009-2011 eFaber, S.L.
// Copyright 2009-2011 Ejie, S.A.
// Copyrigth 2009-2011 Dirección de Gobierno Abierto y Comunicación en Internet; 
//    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
//    Gobierno Vasco – Eusko Jaurlaritza 
// Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
// por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
// Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
// copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
// Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
// el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
// SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
// Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
// que establece la Licencia
//
//  http://open.irekia.net, openirekia@efaber.net

package net.efaber.irekia.social.comment;

import android.os.AsyncTask;
import android.util.Log;
import com.github.droidfu.http.BetterHttp;
import com.github.droidfu.http.BetterHttpResponse;
import net.efaber.irekia.I18n;
import net.efaber.irekia.Irekia;
import net.efaber.irekia.JSON;
import net.efaber.irekia.social.PostResult;
import net.efaber.irekia.social.Social_Activity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.params.CookiePolicy;
import org.apache.http.cookie.CookieSpecRegistry;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.cookie.*;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.net.ConnectException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * Class in charge of fetching the child JSON data from the server.
 */
public class CommentTask extends AsyncTask<String, Integer, PostResult> {
	private static final String TAG = "Irekia.CommentTask";

	private final String comment_url;
	private final String comment;
	private final int item_id;
	private final int controller_id;
	private final Social_Activity activity;

	public CommentTask(String comment_url, String comment, int item_id, int controller_id, Social_Activity activity) {
		super();
		this.comment_url = comment_url.replace("_CONTROLLER_ID_", String.valueOf(controller_id))
			.replace("_LNG_", I18n.get_effective_language())
			.replace("_ITEM_ID_", String.valueOf(item_id));
		this.comment = comment;
		this.controller_id = controller_id;
		this.item_id = item_id;
		this.activity = activity;
	}

	/**
	 * Downloads and processes the retrieved JSON.
	 */
	protected PostResult doInBackground(String... urls) {

		try {
			//if device has network available ask for new items
			if (Irekia.isNetworkAvailable() && Irekia.main_Overlord != null) {
				if (Irekia.user_auth_info == null || !Irekia.user_auth_info.is_Logged())
					return PostResult.NEEDS_AUTH;
				Irekia.cookieSyncManager.sync();
				if (Irekia.cookieManager.hasCookies()) {
					String c = Irekia.cookieManager.getCookie(new URL(Irekia.main_Overlord.auth_info_url.replace("_LNG_", I18n.get_effective_language())).getHost());
					if (c != null) {
						BasicCookieStore cookieStore = new BasicCookieStore();
						String[] cookieParts = null;
						String cookies[] = null;

						cookies = c.split(";");

						for (int i = 0; i < cookies.length; i++) {
							cookieParts = cookies[i].split("=");
							BasicClientCookie sessionCookie = new BasicClientCookie(cookieParts[0].trim(), cookieParts[1].trim());
							sessionCookie.setDomain(new URL(Irekia.main_Overlord.auth_info_url.replace("_LNG_", I18n.get_effective_language())).getHost());
							cookieStore.addCookie(sessionCookie);
						}
						BetterHttp.getHttpClient().setCookieStore(cookieStore);
						// not sure if I need to add all these specs, but may as well
						CookieSpecRegistry cookie_spec_registry = new CookieSpecRegistry();
						cookie_spec_registry.register(
							CookiePolicy.BEST_MATCH,
							new BestMatchSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.BROWSER_COMPATIBILITY,
							new BrowserCompatSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.NETSCAPE,
							new NetscapeDraftSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.RFC_2109,
							new RFC2109SpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.RFC_2965,
							new RFC2965SpecFactory());
						BetterHttp.getHttpClient().setCookieSpecs(cookie_spec_registry);
					}
				}
				JSONObject comment_inner = new JSONObject();
				comment_inner.put("comment_text", comment);
				comment_inner.put("contoller_id", controller_id);
				comment_inner.put("item_id", item_id);
				List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>(1);
				nameValuePairs.add(new BasicNameValuePair("data", comment_inner.toString()));
				BetterHttpResponse response = BetterHttp.post(comment_url, new UrlEncodedFormEntity(nameValuePairs)).send();
				if (response.getStatusCode() == 200 && !response.getResponseBodyAsString().equals("")) {
					JSON json = JSON.parse(response.getResponseBodyAsString());
					if (json == null)
						return PostResult.ERROR.setMessage(I18n.e(I18n.Embedded_string.CONNECTION_ERROR));
					else if (json.get_bool("requires_moderation", false)) {
						return PostResult.REQUIRES_MODERATION.setMessage(I18n.e(I18n.Embedded_string.COMMENT_MODERATION));
					} else if (json.get_bool("accepted", false)) {
						return PostResult.ACCEPTED.setMessage(I18n.e(I18n.Embedded_string.COMMENT_ACCEPTED));
					} else if (json.get_bool("needs_auth", false)) {
						return PostResult.NEEDS_AUTH;
					} else if (!json.get_string("error_message", "").equals("")) {
						return PostResult.ERROR.setMessage(json.get_string("error_message", ""));
					}
				} else if (response.getStatusCode() == 500)
					return PostResult.ERROR.setMessage(I18n.e(I18n.Embedded_string.CONNECTION_ERROR));
			}
		} catch (ConnectException e) {
			e.printStackTrace();
			Log.w(TAG, "ConnectException error: " + e.getLocalizedMessage());
		} catch (IOException e) {
			e.printStackTrace();
			Log.w(TAG, "IOException error: " + e.getLocalizedMessage());
		} catch (JSONException e) {
			e.printStackTrace();
			Log.w(TAG, "JSONException error: " + e.getLocalizedMessage());
		}
		return PostResult.ERROR.setMessage(I18n.e(I18n.Embedded_string.CONNECTION_ERROR));
	}

	/**
	 * If new_items is valid, it is assigned to the overlord.
	 */
	protected void onPostExecute(PostResult result) {
		activity.cancelDialog();
		switch (result) {
			case ACCEPTED: {
				activity.reload();
				activity.showToast(result.getMessage());
			}
			break;
			case NEEDS_AUTH: {
				activity.needsLogin();
			}
			break;
			case REQUIRES_MODERATION: {
				activity.showToast(result.getMessage());
			}
			break;
			case ERROR: {
				activity.showToast(result.getMessage());
			}
			break;
		}
	}


}
