package net.efaber.irekia.social.create;

import android.app.Activity;
import android.app.AlertDialog;
import net.efaber.irekia.I18n;
import net.efaber.irekia.JSON;
import net.efaber.irekia.social.PostResult;
import net.efaber.irekia.social.Social_Activity;
import net.efaber.irekia.social.create.step.Step;
import net.efaber.irekia.social.create.step.StepType;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import static junit.framework.Assert.assertTrue;

public final class StepController {
	Map<String, Object> steps_vars = new HashMap<String, Object>();
	final String start_point_url;
	final Activity activity;
	final Stack<Step> steps = new Stack<Step>();

	public StepController(String start_point_url, Activity activity) {
		assertTrue("Invalid Activity", activity instanceof Social_Activity);
		this.start_point_url = start_point_url.replace("_LNG_", I18n.get_effective_language());
		this.activity = activity;
	}

	public void start() {
		((Social_Activity) activity).showDialog(I18n.e(I18n.Embedded_string.CONNECTING_TO_SERVER));
		new StepTask(start_point_url, null, this, true).execute();
	}

	public void nextStep(Step previous_step) {
		if (!steps.contains(previous_step))
			steps.push(previous_step);
		steps_vars.putAll(previous_step.getCurrent_step_vars());
		((Social_Activity) activity).showDialog(I18n.e(I18n.Embedded_string.CONNECTING_TO_SERVER));
		new StepTask(previous_step.getStep_post_url(), constructPayload(previous_step.getStep_post_vars()), this, false).execute();
	}

	public void performStep(JSON json, PostResult result) {
		try {
			((Social_Activity) activity).cancelDialog();
			if (result == PostResult.END)
				((Social_Activity) getActivity()).showToast(result.getMessage());
			else if (result == PostResult.ERROR)
				((Social_Activity) getActivity()).showToast(result.getMessage());
			else if (result == PostResult.NEEDS_AUTH)
				((Social_Activity) getActivity()).needsLogin();
			else {
				String step_type = json.get_string("step_type", "");
				Step step = StepType.fromValue(step_type).getStep(json, this);
				step.showDialog(false);
			}
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}

	public void back(AlertDialog dialog) {
		if (steps.empty())
			dialog.dismiss();
		else {
			Step step = steps.pop();
			dialog.dismiss();
			step.showDialog(true);
		}
	}

	public Activity getActivity() {
		return activity;
	}

	private JSONObject constructPayload(List<String> post_vars) {
		JSONObject payload = new JSONObject();
		try {
			for (String var : post_vars)
				payload.put(var, steps_vars.get(var));
			return payload;
		} catch (JSONException e) {
			e.printStackTrace();
			return new JSONObject();
		}
	}

	public Object getVar(String step_var) {
		return steps_vars.get(step_var);
	}
}
