package net.efaber.irekia.social.create.step;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.widget.ArrayAdapter;
import net.efaber.irekia.JSON;
import net.efaber.irekia.R;
import net.efaber.irekia.social.create.StepController;

import java.util.ArrayList;
import java.util.List;

public class ChoiceStep extends Step {


	List<String> step_choice_names = new ArrayList<String>();
	List<String> step_choice_values = new ArrayList<String>();
	int step_choice_selected;
	boolean step_choice_automatic;

	public ChoiceStep(JSON json, StepController stepController) {
		super(json, stepController, StepType.CHOICE);
		step_choice_names = json.get_array("step_choice_names", new ArrayList<String>(), String.class);
		step_choice_values = json.get_array("step_choice_values", new ArrayList<String>(), String.class);
		step_choice_selected = json.get_int("step_choice_selected", -1);
		step_choice_automatic = json.get_bool("step_choice_automatic", false);
	}

	public AlertDialog showDialog(boolean load) {
		return step_choice_automatic ? getAutomaticDialog(load) : getCommonDialog(load);
	}

	public AlertDialog getAutomaticDialog(boolean load) {
		AlertDialog.Builder builder = new AlertDialog.Builder(stepController.getActivity());
		int selected = step_choice_selected;
		String[] list = new String[step_choice_names.size()];
		for (int i = 0; i < this.step_choice_names.size(); i++) {
			list[i] = step_choice_names.get(i);
		}
		if (load) {
			String var = (String) stepController.getVar(step_var);
			selected = step_choice_values.indexOf(var);
		}
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(stepController.getActivity(), android.R.layout.simple_list_item_single_choice, list);
		builder.setSingleChoiceItems(adapter, selected, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				current_step_vars.put(step_var, step_choice_values.get(which));
				stepController.nextStep(ChoiceStep.this);
			}
		});
		builder.setTitle(getTranslatedStringTitle());
		builder.setIcon(R.drawable.ic_menu_new);
		final AlertDialog alertDialog = builder.create();
		alertDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialogInterface) {
				stepController.back(alertDialog);
			}
		});
		alertDialog.getListView().setBackgroundColor(Color.BLACK);
		alertDialog.show();
		return alertDialog;
	}

	public AlertDialog getCommonDialog(boolean load) {
		AlertDialog.Builder builder = new AlertDialog.Builder(stepController.getActivity());
		int selected = step_choice_selected;
		String[] list = new String[step_choice_names.size()];
		for (int i = 0; i < this.step_choice_names.size(); i++) {
			list[i] = step_choice_names.get(i);
		}
		if (load) {
			String var = (String) stepController.getVar(step_var);
			selected = step_choice_values.indexOf(var);
		}
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(stepController.getActivity(), android.R.layout.simple_list_item_single_choice, list);
		builder.setSingleChoiceItems(adapter, selected, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);
			}
		});
		builder.setPositiveButton(getTranslatedNextButton(), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				int selectedPosition = ((AlertDialog) dialog).getListView().getCheckedItemPosition();
				if (selectedPosition == -1) {
					((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
				} else {
					current_step_vars.put(step_var, step_choice_values.get(selectedPosition));
					stepController.nextStep(ChoiceStep.this);
					dialog.dismiss();
				}
			}
		});
		builder.setTitle(getTranslatedStringTitle());
		builder.setIcon(R.drawable.ic_menu_new);
		final AlertDialog alertDialog = builder.create();
		alertDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialogInterface) {
				stepController.back(alertDialog);
			}
		});
		alertDialog.getListView().setBackgroundColor(Color.BLACK);
		alertDialog.show();
		if (!load)
			alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
		return alertDialog;
	}
}
