package net.efaber.irekia.social.create.step;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Handler;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.*;
import net.efaber.irekia.I18n;
import net.efaber.irekia.JSON;
import net.efaber.irekia.R;
import net.efaber.irekia.social.Social_Activity;
import net.efaber.irekia.social.create.StepController;
import net.efaber.irekia.social.create.step.search.SearchTask;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;

public class SearchStep extends Step {

	private String step_search_url;
	private int step_search_delay;
	private int step_search_min_chars;
	private boolean step_search_choice_automatic;
	private String selected_item = "";
	public AlertDialog alertDialog;
	private SearchTask searchTask;


	public SearchStep(JSON json, StepController stepController) {
		super(json, stepController, StepType.SEARCH);
		step_search_url = json.get_url("step_search_url", "");
		step_search_delay = json.get_int("step_search_delay", 200);
		step_search_min_chars = json.get_int("step_search_min_chars", 3);
		step_search_choice_automatic = json.get_bool("step_search_choice_automatic", false);

	}

	public StepController getStepController() {
		return stepController;
	}

	public AlertDialog showDialog(boolean load) {
		final AlertDialog alertDialog = step_search_choice_automatic ? getAutomaticDialog() : getCommonDialog();
		alertDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialogInterface) {
				stepController.back(alertDialog);
			}
		});
		return alertDialog;
	}


	public AlertDialog getCommonDialog() {
		((Social_Activity) stepController.getActivity()).cancelDialog();
		final AlertDialog alertDialog = new AlertDialog.Builder(stepController.getActivity()).create();
		alertDialog.setCancelable(true);
		LayoutInflater layoutInflater = stepController.getActivity().getLayoutInflater();
		final View view = layoutInflater.inflate(R.layout.search_dialog, null);
		final ListView lv = (ListView) view.findViewById(R.id.choices_list);
		String[] choices = {};
		lv.setAdapter(new ArrayAdapter<String>(stepController.getActivity(), android.R.layout.simple_list_item_single_choice, choices));
		((TextView) view.findViewById(R.id.status_text)).setText(getTranslatedStringLine());
		EditText searchInput = (EditText) view.findViewById(R.id.search_src_text);
		searchInput.setHint(I18n.e(I18n.Embedded_string.SEARCH_PLACEHOLDER));
		searchInput.addTextChangedListener(new TextWatcher() {
			public void afterTextChanged(final Editable s) {
				selected_item = "";
				if (s.toString().trim().length() >= step_search_min_chars) {
					final Handler handler = new Handler();
					handler.postDelayed(new Runnable() {
						public void run() {
							try {
								alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
								if (searchTask == null || searchTask.getStatus() == AsyncTask.Status.FINISHED) {
									view.findViewById(R.id.items_loading_spinner).setVisibility(View.VISIBLE);
									searchTask = new SearchTask(step_search_url.replace("_LNG_", I18n.get_effective_language())
										.replace("_WORDS_", URLEncoder.encode(s.toString(), "UTF-8")), SearchStep.this);
									searchTask.execute();
								} else {
									searchTask.cancel(true);
									view.findViewById(R.id.items_loading_spinner).setVisibility(View.VISIBLE);
									searchTask = new SearchTask(step_search_url.replace("_LNG_", I18n.get_effective_language())
										.replace("_WORDS_", URLEncoder.encode(s.toString(), "UTF-8")), SearchStep.this);
									searchTask.execute();
								}
							} catch (UnsupportedEncodingException e) {
								e.printStackTrace();
							} catch (IOException e) {
								e.printStackTrace();
							}
						}
					}, step_search_delay);
				} else {
					view.findViewById(R.id.items_loading_spinner).setVisibility(View.GONE);
					lv.setAdapter(new ArrayAdapter<String>(stepController.getActivity(), android.R.layout.simple_list_item_single_choice, new ArrayList<String>()));
				}
			}

			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}

			public void onTextChanged(CharSequence s, int start, int before, int count) {
			}
		});
		alertDialog.setView(view);
		alertDialog.setTitle(getTranslatedStringTitle());
		alertDialog.setIcon(R.drawable.ic_menu_new);
		this.alertDialog = alertDialog;
		this.alertDialog.setButton(AlertDialog.BUTTON_POSITIVE, getTranslatedNextButton(), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				current_step_vars.put(step_var, selected_item);
				stepController.nextStep(SearchStep.this);
			}
		});
		lv.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> arg0, View view, int arg2, long itemID) {
				CheckedTextView item = (CheckedTextView) view.findViewById(android.R.id.text1);
				selected_item = item.getText().toString();
				alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);
			}
		});
		this.alertDialog.show();
		this.alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
		return this.alertDialog;
	}

	public AlertDialog getAutomaticDialog() {
		((Social_Activity) stepController.getActivity()).cancelDialog();
		AlertDialog.Builder builder = new AlertDialog.Builder(stepController.getActivity());
		builder.setCancelable(true);
		LayoutInflater layoutInflater = stepController.getActivity().getLayoutInflater();
		final View view = layoutInflater.inflate(R.layout.search_dialog, null);
		final ListView lv = (ListView) view.findViewById(R.id.choices_list);
		String[] choices = {};
		lv.setAdapter(new ArrayAdapter<String>(stepController.getActivity(), android.R.layout.simple_list_item_single_choice, choices));
		((TextView) view.findViewById(R.id.status_text)).setText(getTranslatedStringLine());
		EditText searchInput = (EditText) view.findViewById(R.id.search_src_text);
		searchInput.setHint(I18n.e(I18n.Embedded_string.SEARCH_PLACEHOLDER));
		searchInput.addTextChangedListener(new TextWatcher() {
			public void afterTextChanged(final Editable s) {
				selected_item = "";
				if (s.toString().trim().length() >= step_search_min_chars) {
					final Handler handler = new Handler();
					handler.postDelayed(new Runnable() {
						public void run() {
							try {
								if (searchTask == null || searchTask.getStatus() == AsyncTask.Status.FINISHED) {
									view.findViewById(R.id.items_loading_spinner).setVisibility(View.VISIBLE);
									searchTask = new SearchTask(step_search_url.replace("_LNG_", I18n.get_effective_language())
										.replace("_WORDS_", URLEncoder.encode(s.toString(), "UTF-8")), SearchStep.this);
									searchTask.execute();
								} else {
									searchTask.cancel(true);
									view.findViewById(R.id.items_loading_spinner).setVisibility(View.VISIBLE);
									searchTask = new SearchTask(step_search_url.replace("_LNG_", I18n.get_effective_language())
										.replace("_WORDS_", URLEncoder.encode(s.toString(), "UTF-8")), SearchStep.this);
									searchTask.execute();
								}
							} catch (UnsupportedEncodingException e) {
								e.printStackTrace();
							} catch (IOException e) {
								e.printStackTrace();
							}
						}
					}, step_search_delay);
				} else {
					view.findViewById(R.id.items_loading_spinner).setVisibility(View.GONE);
					lv.setAdapter(new ArrayAdapter<String>(stepController.getActivity(), android.R.layout.simple_list_item_single_choice, new ArrayList<String>()));
				}
			}

			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}

			public void onTextChanged(CharSequence s, int start, int before, int count) {
			}
		});
		builder.setView(view);
		builder.setTitle(getTranslatedStringTitle());
		builder.setIcon(R.drawable.ic_menu_new);
		this.alertDialog = builder.create();
		lv.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> arg0, View view, int arg2, long itemID) {
				CheckedTextView item = (CheckedTextView) view.findViewById(android.R.id.text1);
				selected_item = item.getText().toString();
				alertDialog.dismiss();
				current_step_vars.put(step_var, selected_item);
				stepController.nextStep(SearchStep.this);
			}
		});
		this.alertDialog.show();
		return this.alertDialog;
	}
}
