package net.efaber.irekia.social.create.step;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextWatcher;
import android.view.Gravity;
import android.widget.EditText;
import android.widget.LinearLayout;
import net.efaber.irekia.JSON;
import net.efaber.irekia.R;
import net.efaber.irekia.social.Social_Activity;
import net.efaber.irekia.social.create.StepController;

public class TextStep extends Step {
	int step_text_char_min;
	int step_text_char_max;

	public TextStep(JSON json, StepController stepController) {
		super(json, stepController, StepType.TEXT);
		step_text_char_min = json.get_int("step_text_char_min", 0);
		step_text_char_max = json.get_int("step_text_char_max", 0);
	}

	public AlertDialog showDialog(boolean load) {
		((Social_Activity) stepController.getActivity()).cancelDialog();
		final AlertDialog alertDialog = new AlertDialog.Builder(stepController.getActivity()).create();
		alertDialog.setCancelable(true);
		final EditText input = new EditText(stepController.getActivity());
		input.addTextChangedListener(new TextWatcher() {
			@Override
			public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
			}

			@Override
			public void afterTextChanged(Editable editable) {
				alertDialog.setMessage(getTranslatedStringLine() + "  (" + String.valueOf(step_text_char_max - input.getText().length()) + ")");
				if (input.getText().toString().trim().length() == 0 || (step_text_char_min > 0 && input.getText().length() < step_text_char_min)) {
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);

				} else {
					alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);
				}
			}
		});
		input.setPadding(10, 10, 10, 10);
		input.setGravity(Gravity.TOP);
		LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
		params.setMargins(15, 15, 15, 15);
		input.setLayoutParams(params);
		input.setLines(4);
		LinearLayout layout = new LinearLayout(stepController.getActivity());
		layout.addView(input);
		if (step_text_char_max > 0)
			input.setFilters(new InputFilter[]{new InputFilter.LengthFilter(step_text_char_max)});
		alertDialog.setView(layout);
		alertDialog.setButton(AlertDialog.BUTTON_POSITIVE, getTranslatedNextButton(), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				current_step_vars.put(step_var, input.getText().toString());
				stepController.nextStep(TextStep.this);
			}
		});
		alertDialog.setTitle(getTranslatedStringTitle());
		alertDialog.setMessage(getTranslatedStringLine() + "  (" + step_text_char_max + ")");
		alertDialog.setIcon(R.drawable.ic_menu_new);
		alertDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialogInterface) {
				stepController.back(alertDialog);
			}
		});
		alertDialog.show();
		if (load)
			input.setText((String) stepController.getVar(step_var));
		if (!load)
			alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
		return alertDialog;
	}
}
