//
// OpenIrekia v2.0 Cliente Android
//
// Copyright 2009-2011 eFaber, S.L.
// Copyright 2009-2011 Ejie, S.A.
// Copyrigth 2009-2011 Dirección de Gobierno Abierto y Comunicación en Internet; 
//    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
//    Gobierno Vasco – Eusko Jaurlaritza 
// Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
// por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
// Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
// copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
// Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
// el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
// SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
// Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
// que establece la Licencia
//
//  http://open.irekia.net, openirekia@efaber.net

package net.efaber.irekia.social.create.step.search;

import android.os.AsyncTask;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import com.github.droidfu.http.BetterHttp;
import com.github.droidfu.http.BetterHttpResponse;
import net.efaber.irekia.I18n;
import net.efaber.irekia.Irekia;
import net.efaber.irekia.JSON;
import net.efaber.irekia.R;
import net.efaber.irekia.social.create.step.SearchStep;
import org.apache.http.client.params.CookiePolicy;
import org.apache.http.cookie.CookieSpecRegistry;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.cookie.*;

import java.io.IOException;
import java.net.ConnectException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * Class in charge of fetching the child JSON data from the server.
 */
public class SearchTask extends AsyncTask<String, Integer, Boolean> {
	private static final String TAG = "Irekia.SearchTask";

	private final String search_url;
	SearchStep searchStep;
	private List<String> items = new ArrayList<String>();
	private String empty_message = "";

	public SearchTask(String step_url, SearchStep searchStep) {
		super();
		this.search_url = step_url;
		this.searchStep = searchStep;
	}

	protected Boolean doInBackground(String... urls) {

		try {
			//if device has network available ask for new items
			if (Irekia.isNetworkAvailable() && Irekia.main_Overlord != null) {
				Irekia.cookieSyncManager.sync();
				if (Irekia.cookieManager.hasCookies()) {
					String c = Irekia.cookieManager.getCookie(new URL(Irekia.main_Overlord.auth_info_url.replace("_LNG_", I18n.get_effective_language())).getHost());
					if (c != null) {
						BasicCookieStore cookieStore = new BasicCookieStore();
						String[] cookieParts = null;
						String cookies[] = null;

						cookies = c.split(";");

						for (int i = 0; i < cookies.length; i++) {
							cookieParts = cookies[i].split("=");
							BasicClientCookie sessionCookie = new BasicClientCookie(cookieParts[0].trim(), cookieParts[1].trim());
							sessionCookie.setDomain(new URL(Irekia.main_Overlord.auth_info_url.replace("_LNG_", I18n.get_effective_language())).getHost());
							cookieStore.addCookie(sessionCookie);
						}
						BetterHttp.getHttpClient().setCookieStore(cookieStore);
						// not sure if I need to add all these specs, but may as well
						CookieSpecRegistry cookie_spec_registry = new CookieSpecRegistry();
						cookie_spec_registry.register(
							CookiePolicy.BEST_MATCH,
							new BestMatchSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.BROWSER_COMPATIBILITY,
							new BrowserCompatSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.NETSCAPE,
							new NetscapeDraftSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.RFC_2109,
							new RFC2109SpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.RFC_2965,
							new RFC2965SpecFactory());
						BetterHttp.getHttpClient().setCookieSpecs(cookie_spec_registry);
					}
				}
				BetterHttpResponse response = BetterHttp.get(search_url).send();
				byte[] responseBodyAsBytes = response.getResponseBodyAsBytes();
				JSON json = JSON.parse(new String(responseBodyAsBytes, "UTF-8"));
				items = json.get_array("items", new ArrayList<String>(), String.class);
				empty_message = json.get_string("empty_message", "");
				return true;
			} else
				return false;
		} catch (ConnectException e) {
			e.printStackTrace();
			Log.w(TAG, "ConnectException error: " + e.getLocalizedMessage());
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			Log.w(TAG, "IOException error: " + e.getLocalizedMessage());
			return false;
		}
	}

	protected void onPostExecute(Boolean result) {
		if (result) {
			TextView status = (TextView) searchStep.alertDialog.findViewById(R.id.status_text);
			searchStep.alertDialog.findViewById(R.id.items_loading_spinner).setVisibility(View.GONE);
			ListView lv = (ListView) searchStep.alertDialog.findViewById(R.id.choices_list);
			lv.setAdapter(new ArrayAdapter<String>(searchStep.getStepController().getActivity(), android.R.layout.simple_list_item_single_choice, items));
			if (items.size() == 0) {
				status.setText(empty_message);
				status.setVisibility(View.GONE);
			} else {
				status.setText("");
				status.setVisibility(View.GONE);
			}
		}
	}


}
