//
// OpenIrekia v2.0 Cliente Android
//
// Copyright 2009-2011 eFaber, S.L.
// Copyright 2009-2011 Ejie, S.A.
// Copyrigth 2009-2011 Dirección de Gobierno Abierto y Comunicación en Internet; 
//    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
//    Gobierno Vasco – Eusko Jaurlaritza 
// Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
// por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
// Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
// copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
// Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
// el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
// SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
// Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
// que establece la Licencia
//
//  http://open.irekia.net, openirekia@efaber.net

package net.efaber.irekia.zerg;

import android.os.AsyncTask;
import android.util.Log;
import com.github.droidfu.http.BetterHttp;
import com.github.droidfu.http.BetterHttpResponse;
import net.efaber.irekia.I18n;
import net.efaber.irekia.Irekia;
import net.efaber.irekia.JSON;
import net.efaber.irekia.cache.Cache_Manager;
import net.efaber.irekia.login.User_Auth_Info;
import net.efaber.irekia.models.Content_item;
import net.efaber.irekia.models.Content_item_comparator;
import net.efaber.irekia.models.Section_comparator;
import net.efaber.irekia.models.Section_state;
import org.apache.http.client.params.CookiePolicy;
import org.apache.http.cookie.CookieSpecRegistry;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.cookie.*;
import org.codehaus.jackson.map.DeserializationConfig;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.ConnectException;
import java.net.URL;
import java.text.ParseException;
import java.util.*;

import static junit.framework.Assert.assertTrue;

/**
 * Class in charge of fetching the new JSON data from the server.
 * Broodlings are usually spawned by Brood lords. We are taking here some
 * creative license and presume they would fetch JSON to the overlords too.
 */
public class RootDownloaderTask extends AsyncTask<String, Integer, Boolean> {
	private static final String TAG = "Irekia.Broodling";

	ArrayList<Content_item> new_items = new ArrayList<Content_item>();
	ArrayList<Integer> to_delete = new ArrayList<Integer>();
	ArrayList<Section_state> new_sections = new ArrayList<Section_state>();
	Overlord overlord;
	// Local variable to indicate to RootDownloaderTask if should replace or increment root items
	private final boolean increment;

	public RootDownloaderTask(Overlord overlord, boolean increment) {
		super();
		this.overlord = overlord;
		this.increment = increment;
	}

	/**
	 * Call the parent, in a UI safe way.
	 */
	protected void onProgressUpdate(Integer... progress) {
		overlord.onBroodlingUpdate(progress[0]);
	}

	/**
	 * Downloads and processes the retrieved JSON.
	 */
	protected Boolean doInBackground(String... urls) {
		assertTrue("This is meant to use one url", 1 == urls.length);

		try {
			publishProgress(0);
			if (overlord.items.countObservers() < 1)
				return false;

			JSON json;
			//if device has network available ask for new items
			if (Irekia.isNetworkAvailable()) {
				if (Irekia.cookieManager.hasCookies()) {
					String c = Irekia.cookieManager.getCookie(new URL(urls[0]).getHost());
					if (c != null) {
						BasicCookieStore cookieStore = new BasicCookieStore();
						String[] cookieParts = null;
						String cookies[] = null;

						cookies = c.split(";");

						for (int i = 0; i < cookies.length; i++) {
							cookieParts = cookies[i].split("=");
							BasicClientCookie sessionCookie = new BasicClientCookie(cookieParts[0].trim(), cookieParts[1].trim());
							sessionCookie.setDomain(new URL(urls[0]).getHost());

							cookieStore.addCookie(sessionCookie);
						}
						BetterHttp.getHttpClient().setCookieStore(cookieStore);
						// not sure if I need to add all these specs, but may as well
						CookieSpecRegistry cookie_spec_registry = new CookieSpecRegistry();
						cookie_spec_registry.register(
							CookiePolicy.BEST_MATCH,
							new BestMatchSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.BROWSER_COMPATIBILITY,
							new BrowserCompatSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.NETSCAPE,
							new NetscapeDraftSpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.RFC_2109,
							new RFC2109SpecFactory());
						cookie_spec_registry.register(
							CookiePolicy.RFC_2965,
							new RFC2965SpecFactory());
						BetterHttp.getHttpClient().setCookieSpecs(cookie_spec_registry);
					}
				}
				BetterHttpResponse response = BetterHttp.get(urls[0]).send();

				publishProgress(4000);
				json = JSON.parse(response.getResponseBodyAsString());

				if (null != json) {
					return process_json(json, true);
				} else {
					Log.w(TAG, "Couldn't parse json!");
				}
			} else {
				//if device has not network available just parse old items
				return process_json(null, false);
			}
			publishProgress(5000);

		} catch (ConnectException e) {
			e.printStackTrace();
			Log.w(TAG, "ConnectException error: " + e.getLocalizedMessage());
			return process_json(null, false);
		} catch (IOException e) {
			e.printStackTrace();
			Log.w(TAG, "IOException error: " + e.getLocalizedMessage());
		}
		return Boolean.valueOf(false);
	}

	/**
	 * If new_items is valid, it is assigned to the overlord.
	 */

	protected void onPostExecute(Boolean result) {
		overlord.replace(new_items, new_sections, false);
		try {
			List<Content_item> content_items = new ArrayList<Content_item>(overlord.items);
			// Remove superfluous items according to maximum number of allowed items.
			if (content_items != null && content_items.size() > overlord.cache_size) {
				content_items = content_items.subList(0, overlord.cache_size);
			}
			Irekia.mapper.configure(DeserializationConfig.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY, true);
			//save resultant items to json cache
			if (overlord.sections != null && overlord.sections.size() > 0 && content_items.size() > 0)
				Cache_Manager.save_json(String.valueOf(overlord.id), "{\"" + overlord.race_tag + "\":" + Irekia.mapper.writeValueAsString(content_items) + ", \"sections\":" + Irekia.mapper.writeValueAsString(overlord.sections) + "}");
			else if (content_items.size() > 0)
				Cache_Manager.save_json(String.valueOf(overlord.id), "{\"" + overlord.race_tag + "\":" + Irekia.mapper.writeValueAsString(content_items) + "}");
		} catch (IOException e) {
			e.printStackTrace();
			Log.w(TAG, "IOException error: " + e.getLocalizedMessage());
		}
		try {
			//Check if logged
			if (Irekia.main_Overlord.auth_info_url != null) {
				JSON loginJson = null;
				BetterHttpResponse response = BetterHttp.get(Irekia.main_Overlord.auth_info_url.replace("_LNG_", I18n.get_effective_language())).send();
				if (response.getStatusCode() == 200 && !response.getResponseBodyAsString().equals("")) {
					loginJson = JSON.parse(response.getResponseBodyAsString());
				}
				Irekia.user_auth_info = new User_Auth_Info(loginJson);
			}
		} catch (ConnectException e) {
			e.printStackTrace(); 
		} catch (IOException e) {
			e.printStackTrace(); 
		}
		publishProgress(10000);
	}


	/**
	 * Parses the downloaded json for items in the protocol.
	 * This will use race_tag to find the correct entries.
	 */
	protected Boolean process_json(JSON json, boolean has_connection) {
		//check if current request has older items
		if (has_connection)
			overlord.has_older = json.get_bool("has_older", false);
			//if device has not connectivity avoid fetching more items
		else
			overlord.has_older = false;
		//check if current Overlord has been ever cached or not
		boolean json_exist = Cache_Manager.exist_json(String.valueOf(overlord.id));
		new_items = new ArrayList<Content_item>();
		/*Get Saved Items and sections*/
		if (json_exist) {
			JSON old_json = Cache_Manager.load_json(String.valueOf(overlord.id));
			new_sections = process_sections(
				old_json.get_array("sections", null, HashMap.class), overlord.sections);
			ArrayList<Object> saved_items = new ArrayList<Object>();
			ArrayList<Object> items = old_json.get_array(overlord.race_tag,
				null, HashMap.class);
			if (items != null)
				saved_items.addAll(items);
			// Convert the item JSON into objects.
			for (Object item_stub : saved_items) {
				try {
					new_items.add(overlord.race_item_class.getConstructor(JSON.class)
						.newInstance(JSON.map(item_stub)));
				} catch (IllegalArgumentException e) {
					Log.w(TAG, "Ignoring item 1");
				} catch (SecurityException e) {
					Log.w(TAG, "Ignoring item 2");
				} catch (InstantiationException e) {
					Log.w(TAG, "Ignoring item 3");
				} catch (IllegalAccessException e) {
					Log.w(TAG, "Ignoring item 4");
				} catch (InvocationTargetException e) {
					Log.w(TAG, "Ignoring item 5");
				} catch (NoSuchMethodException e) {
					Log.w(TAG, "Ignoring item 6");
				}
			}
		}
		//if device has connectivity add restored items to final list
		if (has_connection) {
			ArrayList<Object> items = json.get_array(overlord.race_tag,
				null, HashMap.class);

			if (null == items || items.size() < 1) {
				//log.i("No items in result?");
				return Boolean.valueOf(false);
			}

			to_delete = json.get_array("to_delete", new ArrayList<Integer>(),
				Integer.class);

			new_sections.addAll(overlord.sections);
			new_sections.addAll(process_sections(
				json.get_array("sections", null, HashMap.class), overlord.sections));

			publishProgress(6000);

			for (Object item_stub : items) {
				try {
					new_items.add(overlord.race_item_class.getConstructor(JSON.class)
						.newInstance(JSON.map(item_stub)));
				} catch (IllegalArgumentException e) {
					Log.w(TAG, "Ignoring item 1");
				} catch (SecurityException e) {
					Log.w(TAG, "Ignoring item 2");
				} catch (InstantiationException e) {
					Log.w(TAG, "Ignoring item 3");
				} catch (IllegalAccessException e) {
					Log.w(TAG, "Ignoring item 4");
				} catch (InvocationTargetException e) {
					Log.w(TAG, "Ignoring item 5");
				} catch (NoSuchMethodException e) {
					Log.w(TAG, "Ignoring item 6");
				}
			}
			new_items.addAll(overlord.items);
			publishProgress(7000);

		}

		if (new_items.size() < 1) {
			new_items = null;
			return Boolean.valueOf(false);
		} else {
			sort_and_purge();
			// Sort sections by identifier.
			Collections.sort(new_sections, new Section_comparator());
			publishProgress(8000);
			associate_items_to_sections(new_sections, new_items);
			publishProgress(9000);
			return Boolean.valueOf(true);
		}

	}

	/**
	 * Goes through the new_items array sorting and purging elements.
	 * - Sort items according to sort_id + id
	 * - Remove items found in to_delete.
	 * - Remove items out of the maximum overlord limit.
	 * - Remove items past their expiration date.
	 */
	protected void sort_and_purge() {
//		Remove Duplicate items
		HashSet<Content_item> hsi = new HashSet<Content_item>();
		hsi.addAll(new_items);
		new_items.clear();
		new_items.addAll(hsi);
		HashSet<Section_state> hss = new HashSet<Section_state>();
		hss.addAll(new_sections);
		new_sections.clear();
		new_sections.addAll(hss);


		// Remove items as requested by the server.
		for (Integer number : to_delete) {
			for (int f = 0; f < new_items.size(); f++) {
				if (number == new_items.get(f).id) {
					new_items.remove(f);
					break;
				}
			}
		}

		// Remove elements whose expiration date was reached.
		final int now = (int) (System.currentTimeMillis() / 1000);
		for (int f = 0; f < new_items.size(); f++) {
			Content_item item = new_items.get(f);
			if (item.expiration_date > 0 && item.expiration_date < now)
				new_items.remove(f--);
		}

		// Sort items by sorting identifier.
		Collections.sort(new_items, new Content_item_comparator());


	}

	/**
	 * Processes the section list JSON.
	 * This will convert the JSONs, filter duplicate entries, copy the state of the
	 * sections currently in use. Any of the parameters can be null.
	 *
	 * @return The list of sections you have to assign to the overlord. Note that
	 *         these still have not filled the appropriate items, call
	 *         associate_items_to_sections() for that. This function will not return null
	 *         but an empty list.
	 */
	protected ArrayList<Section_state> process_sections(
		ArrayList<Object> section_stubs, ArrayList<Section_state> old_sections) {
		ArrayList<Section_state> valid_sections = new ArrayList<Section_state>();
		if (null == section_stubs)
			return valid_sections;

		// Convert the sections without duplicates, use a hash set to check.
		HashSet<Integer> visited_sections = new HashSet<Integer>();
		for (Object section_stub : section_stubs) {
			try {
				Section_state s = new Section_state(JSON.map(section_stub));
				if (visited_sections.contains(s.id))
					continue;
				valid_sections.add(s);
				visited_sections.add(s.id);
			} catch (ParseException e) {
				Log.w(TAG, "Ignoring section 1 " + section_stub);
			}
		}

		// Try to copy the previous collapsed state in case something changes.
		for (Section_state valid_section : valid_sections) {
			Section_state previous = overlord.section_by_id(valid_section.id);
			if (null != previous)
				valid_section.collapsed = previous.collapsed;
		}

		if (valid_sections.size() > 0 && null != old_sections) {
			// Preserve old sections by copying them to the new ones.
			for (Section_state old_section : old_sections) {
				if (!(visited_sections.contains(old_section.id))) {
					valid_sections.add(new Section_state(old_section));
				}
			}
		}

		// Sort sections by identifier.
		Collections.sort(valid_sections, new Section_comparator());

		return valid_sections;
	}

	/**
	 * Goes through the list of items putting them into any of the sections.
	 * Additionally this method will also purge any empty sections.
	 */
	protected void associate_items_to_sections(ArrayList<Section_state> sections,
											   ArrayList<Content_item> items) {
		assertTrue("Sections can't be null, only empty", null != sections);
		assertTrue("Items can't be null, only empty", null != items);

		if (sections.size() > 0 && items.size() > 0) {
			// First build a HashMap of the sections for easier finding.
			HashMap<Integer, Section_state> d = new HashMap<Integer, Section_state>();
			for (Section_state section : sections)
				d.put(section.id, section);

			for (Content_item item : items) {
				if (null == item.section_ids)
					continue;
				for (Integer section_id : item.section_ids) {
					Section_state dest = d.get(section_id);
					if (null != dest)
						dest.items.add(item);
				}
			}
		}

		// Purge empty sections.
		for (int f = 0; f < sections.size(); f++)
			if (sections.get(f).items.size() < 1)
				sections.remove(f--);
	}

}
